--- LinuxBIOSv2.old/src/stream/serial_stream.c	1970-01-01 00:00:00.000000000 +0000
+++ LinuxBIOSv2/src/stream/serial_stream.c	2006-10-06 23:34:48.000000000 +0000
@@ -0,0 +1,92 @@
+#include <console/console.h>
+#include <stdlib.h>
+#include <stddef.h>
+#include <stream/read_bytes.h>
+#include <string.h>
+
+/* if they set the precompressed rom stream, they better have set a type */
+#if CONFIG_PRECOMPRESSED_ROM_STREAM && ((!CONFIG_COMPRESSED_ROM_STREAM) && (!CONFIG_COMPRESSED_ROM_STREAM_NRV2B) && (!CONFIG_COMPRESSED_ROM_STREAM_LZMA))
+#error "You set CONFIG_PRECOMPRESSED_ROM_STREAM but need to set CONFIG_COMPRESSED_ROM_STREAM (implies NRV2B, deprecated) or CONFIG_COMPRESSED_ROM_STREAM_NRV2B or CONFIG_COMPRESSED_ROM_STREAM_LZMA"
+#endif
+
+#if (CONFIG_COMPRESSED_ROM_STREAM) || (CONFIG_COMPRESSED_ROM_STREAM_NRV2B) 
+#define HAVE_UNCOMPRESSER 1
+#include "../lib/nrv2b.c"
+#endif
+
+#if (CONFIG_COMPRESSED_ROM_STREAM_LZMA)
+#if HAVE_UNCOMPRESSER
+#error "You're defining more than one compression type, which is not allowed (of course)"
+#endif
+#define HAVE_UNCOMPRESSER 1
+#include "../lib/lzma.c"
+#endif
+
+#include "../lib/xmodem.c"
+
+/* Blocks of RAM for storing stream data */
+static unsigned char *stream_start = (unsigned char *) 0x02000000;
+static unsigned char *stream_end;
+static unsigned char *temp_start = (unsigned char *) 0x03000000;
+static int stream_max_bytes = 0x00800000;
+
+#if HAVE_UNCOMPRESSER
+static unsigned long uncompress(uint8_t *src, uint8_t *dest)
+{
+#if (CONFIG_COMPRESSED_ROM_STREAM) || (CONFIG_COMPRESSED_ROM_STREAM_NRV2B) 
+	unsigned long ilen;
+	return unrv2b(src, dest, &ilen);
+#endif
+#if (CONFIG_COMPRESSED_ROM_STREAM_LZMA)
+	return ulzma(src, dest);
+#endif
+}
+#endif
+
+int stream_init(void)
+{
+	int len;
+
+	printk_debug("serial_stream: downloading to 0x%08lx; start XMODEM transfer now!\n", temp_start);
+	if ((len = xmodemReceive(temp_start, stream_max_bytes)) <= 0) {
+		printk_debug("\nserial_stream: XMODEM transfer failed\n");
+		return -1;
+	}
+	printk_debug("\nserial_stream: XMODEM transfer complete; %d bytes received\n", len);
+#if HAVE_UNCOMPRESSER
+	printk_debug("serial_stream: uncompressing to 0x%08lx\n", stream_start);
+	len = uncompress((uint8_t *) temp_start, (uint8_t *) stream_start);
+	printk_debug("serial_stream: %d bytes uncompressed\n", len);
+#else
+	printk_debug("serial_stream: copying to 0x%08lx\n", stream_start);
+	memcpy(stream_start, temp_start, len);
+#endif
+	stream_end = stream_start + len - 1;
+	return 0;
+}
+
+void stream_fini(void)
+{
+}
+
+byte_offset_t stream_skip(byte_offset_t count)
+{
+	if ((stream_start + count) > stream_end) {
+		printk_warning("%6d:%s() - overflowed source buffer\n",
+			__LINE__, __FUNCTION__);
+		count = 0;
+		if (stream_start <= stream_end) {
+			count = (stream_end - stream_start) + 1;
+		}
+	}
+	stream_start += count;
+	return count;
+}
+
+byte_offset_t stream_read(void *vdest, byte_offset_t count)
+{
+	unsigned char *old_start = stream_start;
+	count = stream_skip(count);
+	memcpy(vdest, old_start, count);
+	return count;
+}
--- LinuxBIOSv2.old/src/stream/Config.lb	2006-10-06 00:30:22.000000000 +0000
+++ LinuxBIOSv2/src/stream/Config.lb	2006-10-06 22:09:43.000000000 +0000
@@ -2,6 +2,7 @@
 uses CONFIG_IDE_STREAM
 uses CONFIG_FS_STREAM
 uses CONFIG_IDE
+uses CONFIG_SERIAL_STREAM
 
 if CONFIG_ROM_STREAM
   object rom_stream.o
@@ -16,3 +17,7 @@
   object fs_stream.o
   dir fs
 end
+
+if CONFIG_SERIAL_STREAM
+  object serial_stream.o
+end
--- LinuxBIOSv2.old/src/lib/xmodem.c	1970-01-01 00:00:00.000000000 +0000
+++ LinuxBIOSv2/src/lib/xmodem.c	2006-10-06 22:59:44.000000000 +0000
@@ -0,0 +1,207 @@
+/*
+        Copyright 2006 Arastra, Inc.
+        Copyright 2001, 2002 Georges Menie (www.menie.org)
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
+#include <string.h>
+#include <delay.h>
+
+extern void uart8250_tx_byte(unsigned, unsigned char);
+extern int uart8250_can_rx_byte(unsigned);
+extern unsigned char uart8250_rx_byte(unsigned);
+
+static int _inbyte(int msec)
+{
+	while (!uart8250_can_rx_byte(TTYS0_BASE)) {
+		udelay(1000);
+		if (msec-- <= 0)
+			return -1;
+	}
+	return uart8250_rx_byte(TTYS0_BASE);
+}
+
+static void _outbyte(unsigned char c)
+{
+	uart8250_tx_byte(TTYS0_BASE, c);
+}
+
+/* CRC16 implementation acording to CCITT standards */
+
+static const unsigned short crc16tab[256]= {
+	0x0000,0x1021,0x2042,0x3063,0x4084,0x50a5,0x60c6,0x70e7,
+	0x8108,0x9129,0xa14a,0xb16b,0xc18c,0xd1ad,0xe1ce,0xf1ef,
+	0x1231,0x0210,0x3273,0x2252,0x52b5,0x4294,0x72f7,0x62d6,
+	0x9339,0x8318,0xb37b,0xa35a,0xd3bd,0xc39c,0xf3ff,0xe3de,
+	0x2462,0x3443,0x0420,0x1401,0x64e6,0x74c7,0x44a4,0x5485,
+	0xa56a,0xb54b,0x8528,0x9509,0xe5ee,0xf5cf,0xc5ac,0xd58d,
+	0x3653,0x2672,0x1611,0x0630,0x76d7,0x66f6,0x5695,0x46b4,
+	0xb75b,0xa77a,0x9719,0x8738,0xf7df,0xe7fe,0xd79d,0xc7bc,
+	0x48c4,0x58e5,0x6886,0x78a7,0x0840,0x1861,0x2802,0x3823,
+	0xc9cc,0xd9ed,0xe98e,0xf9af,0x8948,0x9969,0xa90a,0xb92b,
+	0x5af5,0x4ad4,0x7ab7,0x6a96,0x1a71,0x0a50,0x3a33,0x2a12,
+	0xdbfd,0xcbdc,0xfbbf,0xeb9e,0x9b79,0x8b58,0xbb3b,0xab1a,
+	0x6ca6,0x7c87,0x4ce4,0x5cc5,0x2c22,0x3c03,0x0c60,0x1c41,
+	0xedae,0xfd8f,0xcdec,0xddcd,0xad2a,0xbd0b,0x8d68,0x9d49,
+	0x7e97,0x6eb6,0x5ed5,0x4ef4,0x3e13,0x2e32,0x1e51,0x0e70,
+	0xff9f,0xefbe,0xdfdd,0xcffc,0xbf1b,0xaf3a,0x9f59,0x8f78,
+	0x9188,0x81a9,0xb1ca,0xa1eb,0xd10c,0xc12d,0xf14e,0xe16f,
+	0x1080,0x00a1,0x30c2,0x20e3,0x5004,0x4025,0x7046,0x6067,
+	0x83b9,0x9398,0xa3fb,0xb3da,0xc33d,0xd31c,0xe37f,0xf35e,
+	0x02b1,0x1290,0x22f3,0x32d2,0x4235,0x5214,0x6277,0x7256,
+	0xb5ea,0xa5cb,0x95a8,0x8589,0xf56e,0xe54f,0xd52c,0xc50d,
+	0x34e2,0x24c3,0x14a0,0x0481,0x7466,0x6447,0x5424,0x4405,
+	0xa7db,0xb7fa,0x8799,0x97b8,0xe75f,0xf77e,0xc71d,0xd73c,
+	0x26d3,0x36f2,0x0691,0x16b0,0x6657,0x7676,0x4615,0x5634,
+	0xd94c,0xc96d,0xf90e,0xe92f,0x99c8,0x89e9,0xb98a,0xa9ab,
+	0x5844,0x4865,0x7806,0x6827,0x18c0,0x08e1,0x3882,0x28a3,
+	0xcb7d,0xdb5c,0xeb3f,0xfb1e,0x8bf9,0x9bd8,0xabbb,0xbb9a,
+	0x4a75,0x5a54,0x6a37,0x7a16,0x0af1,0x1ad0,0x2ab3,0x3a92,
+	0xfd2e,0xed0f,0xdd6c,0xcd4d,0xbdaa,0xad8b,0x9de8,0x8dc9,
+	0x7c26,0x6c07,0x5c64,0x4c45,0x3ca2,0x2c83,0x1ce0,0x0cc1,
+	0xef1f,0xff3e,0xcf5d,0xdf7c,0xaf9b,0xbfba,0x8fd9,0x9ff8,
+	0x6e17,0x7e36,0x4e55,0x5e74,0x2e93,0x3eb2,0x0ed1,0x1ef0
+};
+  
+static unsigned short crc16_ccitt(const void *buf, int len)
+{
+	register int counter;
+	register unsigned short crc = 0;
+	for( counter = 0; counter < len; counter++)
+		crc = (crc<<8) ^ crc16tab[((crc>>8) ^ *(char *)buf++)&0x00FF];
+	return crc;
+}
+
+#define SOH  0x01
+#define STX  0x02
+#define EOT  0x04
+#define ACK  0x06
+#define NAK  0x15
+#define CAN  0x18
+#define CTRLZ 0x1A
+
+#define DLY_1S 1000
+#define MAXRETRANS 25
+
+static int check(int crc, const unsigned char *buf, int sz)
+{
+	if (crc) {
+		unsigned short crc = crc16_ccitt(buf, sz);
+		unsigned short tcrc = (buf[sz]<<8)+buf[sz+1];
+		if (crc == tcrc)
+			return 1;
+	}
+	else {
+		int i;
+		unsigned char cks = 0;
+		for (i = 0; i < sz; ++i) {
+			cks += buf[i];
+		}
+		if (cks == buf[sz])
+		return 1;
+	}
+
+	return 0;
+}
+
+static void flushinput(void)
+{
+	while (_inbyte(((DLY_1S)*3)>>1) >= 0)
+		;
+}
+
+int xmodemReceive(unsigned char *dest, int destsz)
+{
+	unsigned char xbuff[1030]; /* 1024 for XModem 1k + 3 head chars + 2 crc + nul */
+	unsigned char *p;
+	int bufsz, crc = 0;
+	unsigned char trychar = 'C';
+	unsigned char packetno = 1;
+	int i, c, len = 0;
+	int retry, retrans = MAXRETRANS;
+
+	for(;;) {
+		for( retry = 0; retry < 16; ++retry) {
+			if (trychar) _outbyte(trychar);
+			if ((c = _inbyte((DLY_1S)<<1)) >= 0) {
+				switch (c) {
+				case SOH:
+					bufsz = 128;
+					goto start_recv;
+				case STX:
+					bufsz = 1024;
+					goto start_recv;
+				case EOT:
+					flushinput();
+					_outbyte(ACK);
+					return len; /* normal end */
+				case CAN:
+					if ((c = _inbyte(DLY_1S)) == CAN) {
+						flushinput();
+						_outbyte(ACK);
+						return -1; /* canceled by remote */
+					}
+					break;
+				default:
+					break;
+				}
+			}
+		}
+		if (trychar == 'C') { trychar = NAK; continue; }
+		flushinput();
+		_outbyte(CAN);
+		_outbyte(CAN);
+		_outbyte(CAN);
+		return -2; /* sync error */
+
+	start_recv:
+		if (trychar == 'C') crc = 1;
+		trychar = 0;
+		p = xbuff;
+		*p++ = c;
+		for (i = 0;  i < (bufsz+(crc?1:0)+3); ++i) {
+			if ((c = _inbyte(DLY_1S)) < 0) goto reject;
+			*p++ = c;
+		}
+
+		if (xbuff[1] == (unsigned char)(~xbuff[2]) && 
+			(xbuff[1] == packetno || xbuff[1] == (unsigned char)packetno-1) &&
+			check(crc, &xbuff[3], bufsz)) {
+			if (xbuff[1] == packetno)	{
+				register int count = destsz - len;
+				if (count > bufsz) count = bufsz;
+				if (count > 0) {
+					memcpy (&dest[len], &xbuff[3], count);
+					len += count;
+				}
+				++packetno;
+				retrans = MAXRETRANS+1;
+			}
+			if (--retrans <= 0) {
+				flushinput();
+				_outbyte(CAN);
+				_outbyte(CAN);
+				_outbyte(CAN);
+				return -3; /* too many retry error */
+			}
+			_outbyte(ACK);
+			continue;
+		}
+	reject:
+		flushinput();
+		_outbyte(NAK);
+	}
+}
--- LinuxBIOSv2.old/src/config/Options.lb	2006-10-06 00:30:31.000000000 +0000
+++ LinuxBIOSv2/src/config/Options.lb	2006-10-06 21:56:08.000000000 +0000
@@ -629,6 +629,11 @@
 	export always
 	comment "boot image is already compressed" 
 end
+define CONFIG_SERIAL_STREAM
+	default 0
+	export always
+	comment "Download boot image from serial port"
+end
 define CONFIG_FS_STREAM
 	default 0
 	export always
--- LinuxBIOSv2.old/src/lib/uart8250.c	2006-10-06 00:30:31.000000000 +0000
+++ LinuxBIOSv2/src/lib/uart8250.c	2006-10-05 23:10:49.000000000 +0000
@@ -64,6 +64,8 @@
 	outb(0x0, base_port + UART_IER);
 	/* enable fifo's */
 	outb(0x01, base_port + UART_FCR);
+	/* assert DTR and RTS so the other end is happy */
+	outb(0x03, base_port + UART_MCR);
 	/* Set Baud Rate Divisor to 12 ==> 115200 Baud */
 	outb(0x80 | lcs, base_port + UART_LCR);
 	outb(divisor & 0xFF,   base_port + UART_DLL);
